@@ -3,9 +3,7 @@
  *			Runs setgid mail so is able to lock mailboxes
  *			as well. Liblockfile can call this command.
  *
- * Version:	@(#)dotlockfile.c  1.14  17-Jan-2017  miquels@cistron.nl
- *
- *		Copyright (C) Miquel van Smoorenburg 1999-2017
+ *		Copyright (C) Miquel van Smoorenburg and contributors 1999-2021
  *
  *		This program is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU General Public License
@@ -44,7 +42,7 @@ extern int optind;
 
 extern int is_maillock(const char *lockfile);
 extern int lockfile_create_set_tmplock(const char *lockfile,
-			volatile char **tmplock, int retries, int flags);
+			volatile char **tmplock, int retries, int flags, struct __lockargs *);
 
 static volatile char *tmplock;
 static int quiet;
@@ -83,18 +81,22 @@ int set_signal(int sig, void (*handler)(int))
 }
 
 /*
- *	Sleep for an amout of time while regulary checking if
+ *	Sleep for an amount of time while regulary checking if
  *	our parent is still alive.
  */
-int check_sleep(int sleeptime)
+int check_sleep(int sleeptime, int flags)
 {
 	int		i;
+	int		interval = 5;
 	static int	ppid = 0;
 
 	if (ppid == 0) ppid = getppid();
 
-	for (i = 0; i < sleeptime; i += 5) {
-		sleep(5);
+	if (flags & __L_INTERVAL)
+		interval = 1;
+
+	for (i = 0; i < sleeptime; i += interval) {
+		sleep(interval);
 		if (kill(ppid, 0) < 0 && errno == ESRCH)
 			return L_ERROR;
 	}
@@ -166,8 +168,8 @@ void perror_exit(const char *why) {
  */
 void usage(void)
 {
-	fprintf(stderr, "Usage:  dotlockfile -l [-r retries] [-p] <-m|lockfile>\n");
-	fprintf(stderr, "        dotlockfile -l [-r retries] [-p] <-m|lockfile> command args...\n");
+	fprintf(stderr, "Usage:  dotlockfile -l [-r retries] [-i interval] [-p] [-q] <-m|lockfile>\n");
+	fprintf(stderr, "        dotlockfile -l [-r retries] [-i interval] [-p] [-q] <-m|lockfile> [-P] command args...\n");
 	fprintf(stderr, "        dotlockfile -u|-t\n");
 	exit(1);
 }
@@ -175,17 +177,20 @@ void usage(void)
 int main(int argc, char **argv)
 {
 	struct passwd	*pwd;
+	struct __lockargs args = { 0 };
 	gid_t		gid, egid;
 	char		*lockfile = NULL;
 	char		**cmd = NULL;
 	int 		c, r;
 	int		retries = 5;
+	int		interval = 0;
 	int		flags = 0;
 	int		lock = 0;
 	int		unlock = 0;
 	int		check = 0;
 	int		touch = 0;
 	int		writepid = 0;
+	int		passthrough = 0;
 
 	/*
 	 *	Remember real and effective gid, and
@@ -209,7 +214,7 @@ int main(int argc, char **argv)
 	/*
 	 *	Process the options.
 	 */
-	while ((c = getopt(argc, argv, "+qpNr:mluct")) != EOF) switch(c) {
+	while ((c = getopt(argc, argv, "+qpNr:mluci:tP")) != EOF) switch(c) {
 		case 'q':
 			quiet = 1;
 			break;
@@ -256,9 +261,21 @@ int main(int argc, char **argv)
 		case 'c':
 			check = 1;
 			break;
+		case 'i':
+			interval = atoi(optarg);
+			if (interval <= 0 && strcmp(optarg, "0") != 0) {
+				fprintf(stderr, "dotlockfile: -i needs argument >= 0\n");
+				return L_ERROR;
+			}
+			flags |= __L_INTERVAL;
+			args.interval = interval;
+			break;
 		case 't':
 			touch = 1;
 			break;
+		case 'P':
+			passthrough = 1;
+			break;
 		default:
 			usage();
 			break;
@@ -373,7 +390,7 @@ int main(int argc, char **argv)
 	/*
 	 *	No, lock.
 	 */
-	r = lockfile_create_set_tmplock(lockfile, &tmplock, retries, flags);
+	r = lockfile_create_set_tmplock(lockfile, &tmplock, retries, flags, &args);
 	if (r != 0 || !cmd)
 		return r;
 
@@ -431,6 +448,12 @@ int main(int argc, char **argv)
 	alarm(0);
 	lockfile_remove(lockfile);
 
+	if (passthrough) {
+		if (WIFEXITED(wstatus))
+			return WEXITSTATUS(wstatus);
+		if (WIFSIGNALED(wstatus))
+			return 128+WTERMSIG(wstatus);
+	}
 	return 0;
 }
 
