@@ -3,9 +3,7 @@
  *		This file also holds the implementation for
  *		the Svr4 maillock functions.
  *
- * Version:	@(#)lockfile.c  1.06  04-Jun-2004  miquels@cistron.nl
- *
- *		Copyright (C) Miquel van Smoorenburg 1997,1998,1999,2004.
+ *		Copyright (C) Miquel van Smoorenburg and contributors 1997-2021.
  *
  *		This library is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU Library General Public
@@ -21,6 +19,7 @@
 #endif
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -42,7 +41,7 @@ static int  islocked = 0;
 #endif
 
 #ifndef LIB
-extern int check_sleep(int);
+extern int check_sleep(int, int);
 #endif
 
 #ifdef MAILGROUP
@@ -229,7 +228,7 @@ static int lockfilename(const char *lockfile, char *tmplock, int tmplocksz)
 static int lockfile_create_save_tmplock(const char *lockfile,
 		char *tmplock, int tmplocksz,
 		volatile char **xtmplock,
-		int retries, int flags)
+		int retries, int flags, struct __lockargs *args)
 {
 	struct stat	st, st1;
 	char		pidbuf[40];
@@ -241,6 +240,11 @@ static int lockfile_create_save_tmplock(const char *lockfile,
 	int		dontsleep = 1;
 	int		tries = retries + 1;
 
+	/* process optional flags that have arguments */
+	if (flags & __L_INTERVAL) {
+		sleeptime = args->interval;
+	}
+
 	/* decide which PID to write to the lockfile */
 	if (flags & L_PID)
 		pid = getpid();
@@ -289,14 +293,15 @@ static int lockfile_create_save_tmplock(const char *lockfile,
 	 *	Now try to link the temporary lock to the lock.
 	 */
 	for (i = 0; i < tries && tries > 0; i++) {
-
 		if (!dontsleep) {
-			sleeptime += 5;
+			if (!(flags & __L_INTERVAL))
+				sleeptime += 5;
+
 			if (sleeptime > 60) sleeptime = 60;
 #ifdef LIB
 			sleep(sleeptime);
 #else
-			if ((e = check_sleep(sleeptime)) != 0) {
+			if ((e = check_sleep(sleeptime, flags)) != 0) {
 				unlink(tmplock);
 				tmplock[0] = 0;
 				return e;
@@ -314,7 +319,7 @@ static int lockfile_create_save_tmplock(const char *lockfile,
 		 *	EXTRA FIX: the value of the nlink field
 		 *	can't be trusted (may be cached).
 		 */
-		(void)link(tmplock, lockfile);
+		(void)!link(tmplock, lockfile);
 
 		if (lstat(tmplock, &st1) < 0) {
 			tmplock[0] = 0;
@@ -354,7 +359,13 @@ static int lockfile_create_save_tmplock(const char *lockfile,
 		 *	remove the lockfile.
 		 */
 		if (lockfile_check(lockfile, flags) == -1) {
-			unlink(lockfile);
+			if (unlink(lockfile) < 0 && errno != ENOENT) {
+				/*
+				 *	we failed to unlink the stale
+				 *	lockfile, give up.
+				 */
+				return L_RMSTALE;
+			}
 			dontsleep = 1;
 			/*
 			 *	If the lockfile was invalid, then the first
@@ -374,7 +385,7 @@ static int lockfile_create_save_tmplock(const char *lockfile,
 #ifdef LIB
 static
 #endif
-int lockfile_create_set_tmplock(const char *lockfile, volatile char **xtmplock, int retries, int flags)
+int lockfile_create_set_tmplock(const char *lockfile, volatile char **xtmplock, int retries, int flags, struct __lockargs *args)
 {
 	char *tmplock;
 	int l, r, e;
@@ -384,7 +395,7 @@ int lockfile_create_set_tmplock(const char *lockfile, volatile char **xtmplock,
 		return L_ERROR;
 	tmplock[0] = 0;
 	r = lockfile_create_save_tmplock(lockfile,
-						tmplock, l, xtmplock, retries, flags);
+						tmplock, l, xtmplock, retries, flags, args);
 	if (xtmplock)
 		*xtmplock = NULL;
 	e = errno;
@@ -396,8 +407,41 @@ int lockfile_create_set_tmplock(const char *lockfile, volatile char **xtmplock,
 #ifdef LIB
 int lockfile_create(const char *lockfile, int retries, int flags)
 {
-	return lockfile_create_set_tmplock(lockfile, NULL, retries, flags);
+	/* check against unknown flags */
+	if (flags & ~(L_PID|L_PPID)) {
+		errno = EINVAL;
+		return L_ERROR;
+	}
+	return lockfile_create_set_tmplock(lockfile, NULL, retries, flags, NULL);
 }
+
+#ifdef STATIC
+int lockfile_create2(const char *lockfile, int retries,
+		int flags, struct __lockargs *args, int args_sz)
+{
+
+	#define FLAGS_WITH_ARGS (__L_INTERVAL)
+	#define KNOWN_FLAGS (L_PID|L_PPID|__L_INTERVAL)
+
+	/* check if size is the same (version check) */
+	if (args != NULL && sizeof(struct __lockargs) != args_sz) {
+		errno = EINVAL;
+		return L_ERROR;
+	}
+	/* some flags _must_ have a non-null args */
+	if (args == NULL && (flags & FLAGS_WITH_ARGS)) {
+		errno = EINVAL;
+		return L_ERROR;
+	}
+	/* check against unknown flags */
+	if (flags & ~KNOWN_FLAGS) {
+		errno = EINVAL;
+		return L_ERROR;
+	}
+	return lockfile_create_set_tmplock(lockfile, NULL, retries, flags, args);
+}
+#endif
+
 #endif
 
 /*
